
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.043.
use strict;
use warnings;

use Module::Build 0.28;
use lib qw{inc}; use MyBuilder;

my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "execute processes as pipeline",
  "dist_author" => [
    "Shoichi Kaji <skaji\@cpan.org>"
  ],
  "dist_name" => "Process-Pipeline",
  "dist_version" => "0.03",
  "license" => "perl",
  "module_name" => "Process::Pipeline",
  "recursive_test_files" => 1,
  "requires" => {
    "Process::Status" => 0,
    "perl" => "5.008001"
  },
  "test_requires" => {
    "Test::More" => "0.98"
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "Test::More" => "0.98"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = MyBuilder->new(%module_build_args);


$build->create_build_script;
